@@ -5,16 +5,13 @@ from typing import (Callable,
                     Type,
                     cast)
 
+from ground.base import (Context,
+                         Orientation,
+                         Relation)
+from ground.hints import Point
 from prioq.base import PriorityQueue
 from reprit.base import generate_repr
-from robust.angular import (Orientation,
-                            orientation)
-from robust.linear import (SegmentsRelationship,
-                           segments_intersection,
-                           segments_relationship)
 
-from clipping.hints import (Point,
-                            Segment)
 from .enums import OverlapKind
 from .event import (BinaryEvent,
                     Event,
@@ -22,28 +19,29 @@ from .event import (BinaryEvent,
                     MixedEvent,
                     NaryEvent,
                     ShapedEvent)
+from .hints import (Orienteer,
+                    SegmentEndpoints)
 
 
 class BinaryEventsQueueKey:
-    __slots__ = 'event',
+    __slots__ = 'event', 'orienteer'
 
-    def __init__(self, event: BinaryEvent) -> None:
-        self.event = event
+    def __init__(self, orienteer: Orienteer, event: BinaryEvent) -> None:
+        self.orienteer, self.event = orienteer, event
 
     __repr__ = generate_repr(__init__)
 
     def __lt__(self, other: 'BinaryEventsQueueKey') -> bool:
         event, other_event = self.event, other.event
-        start_x, start_y = event.start
-        other_start_x, other_start_y = other_event.start
-        if start_x != other_start_x:
+        start, other_start = event.start, other_event.start
+        if start.x != other_start.x:
             # different x-coordinate,
             # the event with lower x-coordinate is processed first
-            return start_x < other_start_x
-        elif start_y != other_start_y:
+            return start.x < other_start.x
+        elif start.y != other_start.y:
             # different points, but same x-coordinate,
             # the event with lower y-coordinate is processed first
-            return start_y < other_start_y
+            return start.y < other_start.y
         elif event.is_right_endpoint is not other_event.is_right_endpoint:
             # same start, but one is a left endpoint
             # and the other a right endpoint,
@@ -52,8 +50,8 @@ class BinaryEventsQueueKey:
         # same start, both events are left endpoints
         # or both are right endpoints
         else:
-            other_end_orientation = orientation(event.end, event.start,
-                                                other_event.end)
+            other_end_orientation = self.orienteer(event.start, event.end,
+                                                   other_event.end)
             # the lowest segment is processed first
             return (other_event.from_left
                     if other_end_orientation is Orientation.COLLINEAR
@@ -65,25 +63,24 @@ class BinaryEventsQueueKey:
 
 
 class NaryEventsQueueKey:
-    __slots__ = 'event',
+    __slots__ = 'event', 'orienteer'
 
-    def __init__(self, event: NaryEvent) -> None:
-        self.event = event
+    def __init__(self, orienteer: Orienteer, event: NaryEvent) -> None:
+        self.orienteer, self.event = orienteer, event
 
     __repr__ = generate_repr(__init__)
 
     def __lt__(self, other: 'NaryEventsQueueKey') -> bool:
         event, other_event = self.event, other.event
-        start_x, start_y = event.start
-        other_start_x, other_start_y = other_event.start
-        if start_x != other_start_x:
+        start, other_start = event.start, other_event.start
+        if start.x != other_start.x:
             # different x-coordinate,
             # the event with lower x-coordinate is processed first
-            return start_x < other_start_x
-        elif start_y != other_start_y:
+            return start.x < other_start.x
+        elif start.y != other_start.y:
             # different points, but same x-coordinate,
             # the event with lower y-coordinate is processed first
-            return start_y < other_start_y
+            return start.y < other_start.y
         elif event.is_right_endpoint is not other_event.is_right_endpoint:
             # same start, but one is a left endpoint
             # and the other a right endpoint,
@@ -93,29 +90,50 @@ class NaryEventsQueueKey:
         # or both are right endpoints
         else:
             # the lowest segment is processed first
-            return (orientation(event.start, other_event.end, event.end)
+            return (self.orienteer(event.start, event.end, other_event.end)
                     is (Orientation.CLOCKWISE
                         if event.is_right_endpoint
                         else Orientation.COUNTERCLOCKWISE))
 
 
 class LinearBinaryEventsQueue:
-    __slots__ = '_queue',
+    __slots__ = 'context', '_queue'
 
-    def __init__(self) -> None:
-        self._queue = PriorityQueue(key=BinaryEventsQueueKey)
+    def __init__(self, context: Context) -> None:
+        self.context = context
+        self._queue = PriorityQueue(key=partial(BinaryEventsQueueKey,
+                                                context.angle_orientation))
 
     __repr__ = generate_repr(__init__)
 
     def __bool__(self) -> bool:
         return bool(self._queue)
 
+    @property
+    def key(self) -> Callable[[BinaryEvent], BinaryEventsQueueKey]:
+        return self._queue.key
+
     def detect_intersection(self,
                             below_event: BinaryEvent,
                             event: BinaryEvent) -> None:
-        below_segment, segment = below_event.segment, event.segment
-        relationship = segments_relationship(below_segment, segment)
-        if relationship is SegmentsRelationship.OVERLAP:
+        below_segment_start, below_segment_end = (below_event.start,
+                                                  below_event.end)
+        segment_start, segment_end = event.start, event.end
+        relation = self.context.segments_relation(below_segment_start,
+                                                  below_segment_end,
+                                                  segment_start, segment_end)
+        if relation is Relation.CROSS or relation is Relation.TOUCH:
+            if (event.start != below_event.start
+                    and event.end != below_event.end):
+                # segments do not intersect_multipolygons at endpoints
+                point = self.context.segments_intersection(
+                        below_segment_start, below_segment_end, segment_start,
+                        segment_end)
+                if point != below_event.start and point != below_event.end:
+                    self._divide_segment(below_event, point)
+                if point != event.start and point != event.end:
+                    self._divide_segment(event, point)
+        elif relation is not Relation.DISJOINT:
             # segments overlap
             if below_event.from_left is event.from_left:
                 raise ValueError('Segments of the same multisegment '
@@ -123,16 +141,14 @@ class LinearBinaryEventsQueue:
             starts_equal = below_event.start == event.start
             if starts_equal:
                 start_min = start_max = None
-            elif (BinaryEventsQueueKey(event)
-                  < BinaryEventsQueueKey(below_event)):
+            elif self.key(event) < self.key(below_event):
                 start_min, start_max = event, below_event
             else:
                 start_min, start_max = below_event, event
             ends_equal = event.end == below_event.end
             if ends_equal:
                 end_min = end_max = None
-            elif (BinaryEventsQueueKey(event.complement)
-                  < BinaryEventsQueueKey(below_event.complement)):
+            elif self.key(event.complement) < self.key(below_event.complement):
                 end_min, end_max = event.complement, below_event.complement
             else:
                 end_min, end_max = below_event.complement, event.complement
@@ -151,24 +167,15 @@ class LinearBinaryEventsQueue:
                 # no line segment includes the other one
                 self._divide_segment(start_max, end_min.start)
                 self._divide_segment(start_min, start_max.start)
-        elif (relationship is not SegmentsRelationship.NONE
-              and event.start != below_event.start
-              and event.end != below_event.end):
-            # segments do not intersect_multipolygons at endpoints
-            point = segments_intersection(below_segment, segment)
-            if point != below_event.start and point != below_event.end:
-                self._divide_segment(below_event, point)
-            if point != event.start and point != event.end:
-                self._divide_segment(event, point)
 
     def pop(self) -> BinaryEvent:
         return self._queue.pop()
 
-    def register_segments(self,
-                          segments: Iterable[Segment],
-                          from_left: bool) -> None:
+    def register(self,
+                 endpoints: Iterable[SegmentEndpoints],
+                 from_left: bool) -> None:
         events_queue = self._queue
-        for start, end in segments:
+        for start, end in endpoints:
             if start > end:
                 start, end = end, start
             start_event = BinaryEvent(False, start, None, from_left)
@@ -187,10 +194,16 @@ class LinearBinaryEventsQueue:
 
 
 class MixedBinaryEventsQueue:
-    __slots__ = '_queue'
+    __slots__ = 'context', '_queue'
 
-    def __init__(self) -> None:
-        self._queue = PriorityQueue(key=BinaryEventsQueueKey)
+    def __init__(self, context: Context) -> None:
+        self.context = context
+        self._queue = PriorityQueue(key=partial(BinaryEventsQueueKey,
+                                                context.angle_orientation))
+
+    @property
+    def key(self) -> Callable[[MixedEvent], BinaryEventsQueueKey]:
+        return self._queue.key
 
     __repr__ = generate_repr(__init__)
 
@@ -200,9 +213,24 @@ class MixedBinaryEventsQueue:
     def detect_intersection(self,
                             below_event: MixedEvent,
                             event: MixedEvent) -> bool:
-        below_segment, segment = below_event.segment, event.segment
-        relationship = segments_relationship(below_segment, segment)
-        if relationship is SegmentsRelationship.OVERLAP:
+        below_segment_start, below_segment_end = (below_event.start,
+                                                  below_event.end)
+        segment_start, segment_end = event.start, event.end
+        relation = self.context.segments_relation(below_segment_start,
+                                                  below_segment_end,
+                                                  segment_start, segment_end)
+        if relation is Relation.CROSS or relation is Relation.TOUCH:
+            if (event.start != below_event.start
+                    and event.end != below_event.end):
+                # segments do not intersect_multipolygons at endpoints
+                point = self.context.segments_intersection(
+                        below_segment_start, below_segment_end, segment_start,
+                        segment_end)
+                if point != below_event.start and point != below_event.end:
+                    self._divide_segment(below_event, point)
+                if point != event.start and point != event.end:
+                    self._divide_segment(event, point)
+        elif relation is not Relation.DISJOINT:
             # segments overlap
             if below_event.from_left is event.from_left:
                 raise ValueError('Edges of the {geometry} '
@@ -214,16 +242,14 @@ class MixedBinaryEventsQueue:
             starts_equal = below_event.start == event.start
             if starts_equal:
                 start_min = start_max = None
-            elif (BinaryEventsQueueKey(event)
-                  < BinaryEventsQueueKey(below_event)):
+            elif self.key(event) < self.key(below_event):
                 start_min, start_max = event, below_event
             else:
                 start_min, start_max = below_event, event
             ends_equal = event.end == below_event.end
             if ends_equal:
                 end_min = end_max = None
-            elif (BinaryEventsQueueKey(event.complement)
-                  < BinaryEventsQueueKey(below_event.complement)):
+            elif self.key(event.complement) < self.key(below_event.complement):
                 end_min, end_max = event.complement, below_event.complement
             else:
                 end_min, end_max = below_event.complement, event.complement
@@ -243,23 +269,14 @@ class MixedBinaryEventsQueue:
                 # no line segment includes the other one
                 self._divide_segment(start_max, end_min.start)
                 self._divide_segment(start_min, start_max.start)
-        elif (relationship is not SegmentsRelationship.NONE
-              and event.start != below_event.start
-              and event.end != below_event.end):
-            # segments do not intersect_multipolygons at endpoints
-            point = segments_intersection(below_segment, segment)
-            if point != below_event.start and point != below_event.end:
-                self._divide_segment(below_event, point)
-            if point != event.start and point != event.end:
-                self._divide_segment(event, point)
         return False
 
     def pop(self) -> MixedEvent:
         return self._queue.pop()
 
-    def register_segments(self,
-                          segments: Iterable[Segment],
-                          from_left: bool) -> None:
+    def register(self,
+                 segments: Iterable[SegmentEndpoints],
+                 from_left: bool) -> None:
         queue = self._queue
         for start, end in segments:
             interior_to_left = True
@@ -285,35 +302,55 @@ class MixedBinaryEventsQueue:
 
 
 class NaryEventsQueue:
-    __slots__ = '_queue',
+    __slots__ = 'context', '_queue'
 
-    def __init__(self) -> None:
-        self._queue = PriorityQueue(key=NaryEventsQueueKey)
+    def __init__(self, context: Context) -> None:
+        self.context = context
+        self._queue = PriorityQueue(key=partial(NaryEventsQueueKey,
+                                                context.angle_orientation))
 
     __repr__ = generate_repr(__init__)
 
     def __bool__(self) -> bool:
         return bool(self._queue)
 
+    @property
+    def key(self) -> Callable[[NaryEvent], NaryEventsQueueKey]:
+        return self._queue.key
+
     def detect_intersection(self,
                             below_event: NaryEvent,
                             event: NaryEvent) -> None:
-        below_segment, segment = below_event.segment, event.segment
-        relationship = segments_relationship(below_segment, segment)
-        if relationship is SegmentsRelationship.OVERLAP:
+        below_segment_start, below_segment_end = (below_event.start,
+                                                  below_event.end)
+        segment_start, segment_end = event.start, event.end
+        relation = self.context.segments_relation(below_segment_start,
+                                                  below_segment_end,
+                                                  segment_start, segment_end)
+        if relation is Relation.CROSS or relation is Relation.TOUCH:
+            if (event.start != below_event.start
+                    and event.end != below_event.end):
+                # segments do not intersect_multipolygons at endpoints
+                point = self.context.segments_intersection(
+                        below_segment_start, below_segment_end, segment_start,
+                        segment_end)
+                if point != below_event.start and point != below_event.end:
+                    self._divide_segment(below_event, point)
+                if point != event.start and point != event.end:
+                    self._divide_segment(event, point)
+        elif relation is not Relation.DISJOINT:
             # segments overlap
             starts_equal = below_event.start == event.start
             if starts_equal:
                 start_min = start_max = None
-            elif NaryEventsQueueKey(event) < NaryEventsQueueKey(below_event):
+            elif self.key(event) < self.key(below_event):
                 start_min, start_max = event, below_event
             else:
                 start_min, start_max = below_event, event
             ends_equal = event.end == below_event.end
             if ends_equal:
                 end_min = end_max = None
-            elif (NaryEventsQueueKey(event.complement)
-                  < NaryEventsQueueKey(below_event.complement)):
+            elif self.key(event.complement) < self.key(below_event.complement):
                 end_min, end_max = event.complement, below_event.complement
             else:
                 end_min, end_max = below_event.complement, event.complement
@@ -332,22 +369,13 @@ class NaryEventsQueue:
                 # no line segment includes the other one
                 self._divide_segment(start_max, end_min.start)
                 self._divide_segment(start_min, start_max.start)
-        elif (relationship is not SegmentsRelationship.NONE
-              and event.start != below_event.start
-              and event.end != below_event.end):
-            # segments do not intersect_multipolygons at endpoints
-            point = segments_intersection(below_segment, segment)
-            if point != below_event.start and point != below_event.end:
-                self._divide_segment(below_event, point)
-            if point != event.start and point != event.end:
-                self._divide_segment(event, point)
 
     def pop(self) -> NaryEvent:
         return self._queue.pop()
 
-    def register_segments(self, segments: Iterable[Segment]) -> None:
+    def register(self, endpoints: Iterable[SegmentEndpoints]) -> None:
         queue = self._queue
-        for start, end in segments:
+        for start, end in endpoints:
             if start > end:
                 start, end = end, start
             start_event = NaryEvent(False, start, None)
@@ -365,21 +393,41 @@ class NaryEventsQueue:
 
 
 class ShapedBinaryEventsQueue(Generic[Event]):
-    __slots__ = 'event_cls', '_queue'
+    __slots__ = 'context', 'event_cls', '_queue'
 
-    def __init__(self, event_cls: Type[Event]) -> None:
-        self.event_cls = event_cls
-        self._queue = PriorityQueue(key=BinaryEventsQueueKey)
+    def __init__(self, event_cls: Type[Event], context: Context) -> None:
+        self.event_cls, self.context = event_cls, context
+        self._queue = PriorityQueue(key=partial(BinaryEventsQueueKey,
+                                                context.angle_orientation))
 
     __repr__ = generate_repr(__init__)
 
     def __bool__(self) -> bool:
         return bool(self._queue)
 
+    @property
+    def key(self) -> Callable[[Event], BinaryEventsQueueKey]:
+        return self._queue.key
+
     def detect_intersection(self, below_event: Event, event: Event) -> bool:
-        below_segment, segment = below_event.segment, event.segment
-        relationship = segments_relationship(below_segment, segment)
-        if relationship is SegmentsRelationship.OVERLAP:
+        below_segment_start, below_segment_end = (below_event.start,
+                                                  below_event.end)
+        segment_start, segment_end = event.start, event.end
+        relation = self.context.segments_relation(below_segment_start,
+                                                  below_segment_end,
+                                                  segment_start, segment_end)
+        if relation is Relation.CROSS or relation is Relation.TOUCH:
+            if (event.start != below_event.start
+                    and event.end != below_event.end):
+                # segments do not intersect_multipolygons at endpoints
+                point = self.context.segments_intersection(
+                        below_segment_start, below_segment_end, segment_start,
+                        segment_end)
+                if point != below_event.start and point != below_event.end:
+                    self._divide_segment(below_event, point)
+                if point != event.start and point != event.end:
+                    self._divide_segment(event, point)
+        elif relation is not Relation.DISJOINT:
             # segments overlap
             if below_event.from_left is event.from_left:
                 raise ValueError('Edges of the same multipolygon '
@@ -387,16 +435,14 @@ class ShapedBinaryEventsQueue(Generic[Event]):
             starts_equal = below_event.start == event.start
             if starts_equal:
                 start_min = start_max = None
-            elif (BinaryEventsQueueKey(event)
-                  < BinaryEventsQueueKey(below_event)):
+            elif self.key(event) < self.key(below_event):
                 start_min, start_max = event, below_event
             else:
                 start_min, start_max = below_event, event
             ends_equal = event.end == below_event.end
             if ends_equal:
                 end_min = end_max = None
-            elif (BinaryEventsQueueKey(event.complement)
-                  < BinaryEventsQueueKey(below_event.complement)):
+            elif self.key(event.complement) < self.key(below_event.complement):
                 end_min, end_max = event.complement, below_event.complement
             else:
                 end_min, end_max = below_event.complement, event.complement
@@ -420,26 +466,17 @@ class ShapedBinaryEventsQueue(Generic[Event]):
                 # no line segment includes the other one
                 self._divide_segment(start_max, end_min.start)
                 self._divide_segment(start_min, start_max.start)
-        elif (relationship is not SegmentsRelationship.NONE
-              and event.start != below_event.start
-              and event.end != below_event.end):
-            # segments do not intersect_multipolygons at endpoints
-            point = segments_intersection(below_segment, segment)
-            if point != below_event.start and point != below_event.end:
-                self._divide_segment(below_event, point)
-            if point != event.start and point != event.end:
-                self._divide_segment(event, point)
         return False
 
     def pop(self) -> Event:
         return self._queue.pop()
 
-    def register_segments(self,
-                          segments: Iterable[Segment],
-                          from_left: bool) -> None:
+    def register(self,
+                 endpoints: Iterable[SegmentEndpoints],
+                 from_left: bool) -> None:
         event_cls = self.event_cls
         queue = self._queue
-        for start, end in segments:
+        for start, end in endpoints:
             inside_on_left = True
             if start > end:
                 start, end = end, start
@@ -462,7 +499,9 @@ class ShapedBinaryEventsQueue(Generic[Event]):
         self._queue.push(right_event)
 
 
-HolelessEventsQueue = cast(Callable[[], ShapedBinaryEventsQueue[ShapedEvent]],
+HolelessEventsQueue = cast(Callable[[Context],
+                                    ShapedBinaryEventsQueue[ShapedEvent]],
                            partial(ShapedBinaryEventsQueue, ShapedEvent))
-HoleyEventsQueue = cast(Callable[[], ShapedBinaryEventsQueue[HoleyEvent]],
+HoleyEventsQueue = cast(Callable[[Context],
+                                 ShapedBinaryEventsQueue[HoleyEvent]],
                         partial(ShapedBinaryEventsQueue, HoleyEvent))
